@@ -17,6 +17,7 @@ from collections import OrderedDict
 
 from itertools import chain
 
+import hashlib
 import re
 import sys
 import time
@@ -74,6 +75,7 @@ from .datavars import BuilderError
 from .emerge_fetch import EmergeFetcher, EmergeFetcherError
 from calculate.lib.utils.grub import GrubCommand
 from functools import reduce
+from configparser import ConfigParser
 
 
 _ = lambda x: x
@@ -820,6 +822,7 @@ class Builder(Update):
         были изменены динамически например при исправлении "автомагии"
         """
         var_db_path = path.join(builder_path, 'var/db/pkg')
+        meta_suffix_path = 'metadata/md5-cache'
 
         def rebuild_generator():
             pkg_path = self.clVars.Get("builder.cl_builder_pkg_cache_path")
@@ -828,11 +831,13 @@ class Builder(Update):
                 for repo in repository_data:
                     if pkg.repository in repo:
                         pkg.repository = repo
+                        break
                 else:
                     yield f'{pkg.category}/{pkg.pkg}'
+                    continue
 
                 path_to_repo = pathJoin(builder_path, pkg.repository[1])
-                path_to_file = pathJoin(path_to_repo, meta_suffix_path, pkg[0], pkg[1])
+                path_to_file = pathJoin(path_to_repo, meta_suffix_path, pkg.category, pkg.pkg)
                 if not path.exists(path_to_file):
                     continue
                 c = ConfigParser()
@@ -845,13 +850,18 @@ class Builder(Update):
                                       'RDEPEND': c['data'].get('RDEPEND'),
                                       'PDEPEND': c['data'].get('PDEPEND')}).encode('UTF-8'))
                 new_hash = md.hexdigest()
-                if path.exists(pathJoin(builder_path, pkg_path, meta_suffix_path, pkg[0], pkg[1])):
-                    with open(pathJoin(builder_path, pkg_path, meta_suffix_path, pkg[0], pkg[1])) as f:
+                if path.exists(pathJoin(pkg_path, meta_suffix_path, pkg.repository[0], pkg.category, pkg.pkg)):
+                    with open(pathJoin(pkg_path, meta_suffix_path, pkg.repository[0], pkg.category, pkg.pkg)) as f:
                         old_hash = f.read().strip()
-                with writeFile(pathJoin(builder_path, pkg_path, meta_suffix_path, pkg[0], pkg[1])) as f:
-                    f.write(md.hexdigest())
+                try:
+                    with writeFile(pathJoin(pkg_path, meta_suffix_path, pkg.repository[0], pkg.category, pkg.pkg)) as f:
+                        f.write(md.hexdigest())
+                except PermissionError:
+                    self.printWARNING(_("Failed to write md5 cache: permission error on %s") %
+                                        pathJoin(pkg_path, meta_suffix_path, pkg.repository[0], pkg.category, pkg.pkg))
+                    return []
                 if old_hash and old_hash != new_hash:
-                    yield f'{pkg[0]}/{pkg[1]}'
+                    yield f'{pkg.category}/{pkg.pkg}'
 
         return ["=%s" % x for x in rebuild_generator()]
 
